@@ -52,11 +52,11 @@ DataType = namedtuple("DataType",
 tdsDataTypes = dict(enumerate((
     DataType('tdsTypeVoid', None, 0, None),
     DataType('tdsTypeI8', 'b', 1, np.int8),
-    DataType('tdsTypeI16', 'i', 2, np.int16),
+    DataType('tdsTypeI16', 'h', 2, np.int16),
     DataType('tdsTypeI32', 'l', 4, np.int32),
     DataType('tdsTypeI64', 'q', 8, np.int64),
     DataType('tdsTypeU8', 'B', 1, np.uint8),
-    DataType('tdsTypeU16', 'I', 2, np.uint16),
+    DataType('tdsTypeU16', 'H', 2, np.uint16),
     DataType('tdsTypeU32', 'L', 4, np.uint32),
     DataType('tdsTypeU64', 'Q', 8, np.uint64),
     DataType('tdsTypeSingleFloat', 'f', 4, np.single),
@@ -291,8 +291,13 @@ class _TdmsSegment(object):
         self.data_position = self.position + lead_size + self.raw_data_offset
         if self.next_segment_offset == struct.unpack('<Q', b'\xFF' * 8)[0]:
             # This can happen if Labview crashes
-            log.warning("Last segment of file has unknown size")
+            log.warning("Last segment of file has unknown size, "
+                "not attempting to read it")
             self.next_segment_pos = None
+            self.next_segment_offset = None
+            # Could try to read as much as possible but for now
+            # don't attempt to read last segment
+            raise EOFError
         else:
             self.next_segment_pos = (self.position +
                     self.next_segment_offset + lead_size)
@@ -307,7 +312,7 @@ class _TdmsSegment(object):
             try:
                 self.ordered_objects = previous_segment.ordered_objects
             except AttributeError:
-                raise ValueError("kTocMetaData is set for segment but "
+                raise ValueError("kTocMetaData is not set for segment but "
                         "there is no previous segment")
             self.calculate_chunks()
             return
@@ -620,7 +625,8 @@ class _TdmsSegmentObject(object):
             self.data_size = 0
         # Data has same structure as previously
         elif self.raw_data_index == 0x00000000:
-            log.debug("Object has same data structure as in the previous segment")
+            log.debug("Object has same data structure "
+                    "as in the previous segment")
             pass
         else:
             self.has_data = True
@@ -628,13 +634,22 @@ class _TdmsSegmentObject(object):
 
             # Read the data type
             s = f.read(4)
-            self.data_type = tdsDataTypes[struct.unpack("<L", s)[0]]
+            try:
+                self.data_type = tdsDataTypes[struct.unpack("<L", s)[0]]
+            except KeyError:
+                raise KeyError("Unrecognised data type")
             if (self.tdms_object.data_type is not None and
                     self.data_type != self.tdms_object.data_type):
                 raise ValueError("Segment object doesn't have the same data "
                         "type as previous segments.")
             else:
                 self.tdms_object.data_type = self.data_type
+            log.debug("Object data type: %s" % self.tdms_object.data_type.name)
+
+            if (self.tdms_object.data_type.length is None and
+                    self.tdms_object.data_type.name != 'tdsTypeString'):
+                raise ValueError("Unsupported data type: %s" %
+                        self.tdms_object.data_type.name)
 
             # Read data dimension
             s = f.read(4)
@@ -691,6 +706,8 @@ class _TdmsSegmentObject(object):
             dtype = (np.dtype(self.data_type.nptype).
                     newbyteorder(endianness))
             return np.fromfile(file, dtype=dtype, count=self.number_values)
+        elif self.data_type.name == "tdsTypeString":
+            return read_string_data(file, self.number_values)
         data = self._new_segment_data()
         for i in range(self.number_values):
             data[i] = read_type(file, self.data_type, endianness)
@@ -703,3 +720,21 @@ class _TdmsSegmentObject(object):
             return np.zeros(self.number_values, dtype=self.data_type.nptype)
         else:
             return [None] * self.number_values
+
+
+def read_string_data(file, number_values):
+    """ Read string raw data
+
+        This is stored as an array of offsets
+        followed by the contiguous string data.
+    """
+    offsets = [0]
+    for i in range(number_values):
+        s = file.read(4)
+        offset = struct.unpack("<L", s)[0]
+        offsets.append(offset)
+    strings = []
+    for i in range(number_values):
+        s = file.read(offsets[i + 1] - offsets[i])
+        strings.append(s.decode('utf-8'))
+    return strings
